/**
 * @file
 */

((self, {debug, version, cache}, console) => {

  workbox.setConfig({debug: debug});

  // Keep this in the main file so that autoloading works as expected.
  const {registerRoute, NavigationRoute, setCatchHandler, setDefaultHandler} = workbox.routing;
  const {CacheFirst, CacheOnly, NetworkFirst, NetworkOnly, StaleWhileRevalidate} = workbox.strategies;
  const {precacheAndRoute, matchPrecache} = workbox.precaching;
  const strategies = workbox.strategies;

  // Never cache the serviceworker file or the serviceworker phone home URL.
  const exclude = cache.exclude
    // Make sure we're starting from the begining of the URL to match paths.
    .map(pattern => new RegExp(pattern));

  // Only precache the offline page because the behavior is too aggressive for
  // regular pages like the homepage.
  const precache = []
    .concat(cache.precache.page)
    .concat(cache.precache.asset);

  precacheAndRoute(precache.map(
    url => ({url: url, revision: version})
  ));

  // Set a fetch listener that will enforce exclude URLs for pages and
  // assets as workbox can only exclude pages, not assets.
  self.addEventListener('fetch' , (event) => {
    if (event.request.method === 'GET') {
      const url = new URL(event.request.url);
      const regexExclude = exclude.find(regex => regex.test(url.pathname));
      // Return here to prevent the rest of the processing.
      if (regexExclude) {
        console.log('Exclude Path:', url.pathname, 'pathname matches exclude regex:', regexExclude);
        event.respondWith(fetch(event.request));
      }
    }
  });

  /**
   * Set a default handler for failed requests.
   *
   * This allow the offline page as well as the offline image fallback to work.
   */
  setCatchHandler(({url, event, params}) => {
    const fallback = {
      // Fallback for offline HTML Page
      document: cache.offline.page,
      // Fallback for images
      image: cache.offline.image,
    };

    if (event.request.destination in fallback) {
      return matchPrecache(fallback[event.request.destination]);
    }
  });

  // Allow custom cache patterns, in the config UI:
  // strategy|regex
  cache.patterns.page.forEach(urlConfig => {
    if (urlConfig.strategy in strategies) {
      registerRoute(new NavigationRoute(
        new strategies[urlConfig.strategy]({
          cacheName: 'page-custom',
        }), {
          allowlist: [new RegExp(urlConfig.value)],
          denylist: exclude,
        }
      ));
    }
  });

  // Default strategy for 'page' cache.
  registerRoute(new NavigationRoute(
    new NetworkFirst({
      cacheName: 'page-default',
    }), {
      denylist: exclude,
    }
  ));

  // Define a custom plugin that will enforce size restrictions.
  function cachePlugin({external, maxSize, limitMaxSize}, destination) {
    const responseMaxSize = parseInt(maxSize, 10);
    return {
      /**
       * Custom plugin to enforce caching restrictions.
       *
       * @param request
       * @param response
       * @param event
       * @returns {Promise<null|*>}
       */
      cacheWillUpdate: async ({request, response, event}) => {
        // If cache for external asset is enabled, cache everything.
        if (external && response.status === 0) {
          return response;
        }
        // There has been an error of some kind during the request, do
        // not cache.
        if (response.status !== 200) {
          return null;
        }
        if (limitMaxSize) {
          // The response is fine and we can access the content of the
          // request headers, check the size.
          const size = parseInt(response.headers.get('Content-Length'), 10);
          if ((size && responseMaxSize) && size <= responseMaxSize * 1000) {
            return response;
          }
          console.log(`The response size of ${Math.round(size / 1000)}kB is above the configured limit (${responseMaxSize}kB) of the '${destination}' asset type.`, request.url);
          return null;
        }
        // If there is no size limit, cache everything.
        return response;
      }
    };
  }

  // Asset caching.
  for (const [destination, config] of Object.entries(cache.assets)) {
    if (config.strategy in strategies) {
      registerRoute(
        ({request}) => request.destination === destination,
        new strategies[config.strategy]({
          cacheName: 'asset-default',
          plugins: [cachePlugin(config, destination)],
        })
      );
    }
  }

})(self, drupalPWASettings, pwaConsole);
