/**
 * @file
 * Allow the serviceworker to self remove when the module is disabled.
 */

// Phone-home happens periodically to ensure it is uninstalled when permissions
// changes (if a user login or logout) and when the module is deactivated.
((self, navigator, {debug, phonehome: {idle, count}}, console) => {

  // When the Service Worker goes idle it will reset the variable.
  const PHONE_HOME_CHECK = {
    // Timestamp of the last request.
    latestRequest: Date.now(),
    // Number of page changes since last phone home.
    pageCount: 0,
  };

  // The cache should be assumed to be active by default. After uninstallation has
  // successfully occurred we will set this to false in order to prevent certain
  // conditions where the cache was deleted before new assets were added afterwards.
  let CACHE_ACTIVE = true;

  /**
   * Prevent Phone-home from triggering on every requests.
   *
   * Phome home will be triggered when:
   *  - 5min have passed since the last network request
   *  - 10 pages have been navigated to by the user
   *  - The user has logged in (POST request of a user_login* form)
   *
   * @param {Event} event
   *
   * @return {boolean}
   */
  function doPhoneHomeCheck(event) {
    if (!navigator.onLine) {
      console.debug('Phone-home - Network not detected.');
      return false;
    }

    const now = Date.now();
    // Only count page change, not requests to assets or third party.
    if (event.type === 'fetch' && event.request.mode === 'navigate') {
      PHONE_HOME_CHECK.pageCount += 1;
    }

    // If the user has been idle for 10 minutes, Phone home.
    const timeBased = idle ? (now - PHONE_HOME_CHECK.latestRequest) > idle * 1000 : true;
    // If the user navigates 10 pages, Phone home.
    const countBased = count > 0 ? PHONE_HOME_CHECK.pageCount >= count : true;

    // Update time of last request for any request.
    PHONE_HOME_CHECK.latestRequest = now;
    PHONE_HOME_CHECK.pageCount %= 10;

    return timeBased || countBased;
  }

  /**
   * Phone home
   *
   * Check and see if the Drupal module still exists. The module specifies a
   * dedicated path and when the module is disabled or uninstalled, the URL
   * will 404, signalling to the SW that it needs to unregister itself.
   */
  function phoneHome(event) {
    // Avoid constant phoning-home. Check every 10 pages or if 10 min have
    // passed.
    if (!doPhoneHomeCheck(event)) {
      return;
    }

    // Fetch phone-home URL and process response.
    fetch('/pwa/serviceworker/check')
      .then(response => {
        switch (response.status) {
          case 200:
            console.debug('Phone-home - Network detected, module detected.');
            break;

          case 404:
            console.log('Phone-home - Network detected, module NOT detected. UNINSTALLING.');
            pwaUninstallServiceWorker();
            break;

          case 403:
            console.log('Phone-home - Network detected, current user does not have the permission to use the serviceworker. UNINSTALLING.');
            pwaUninstallServiceWorker();
            break;

          default:
            console.debug('Phone-home - Unexpected response', response);
            break;
        }
      })
      .catch(error => {
        console.error('Phone-home - Error', error);
      });
  }

  /**
   * Uninstall Service Worker
   */
  function pwaUninstallServiceWorker() {
    CACHE_ACTIVE = false;
    return self.registration.unregister()
      .then(success => {
        if (success) {
          // There may be requests already in process before we set CACHE_ACTIVE,
          // defer deleting caches to give a chance for the requests to finish.
          // Otherwise there might be caches set after we clean up the caches.
          setTimeout(() => {
            // Delete all Caches that belong to the PWA module.
            caches.keys().then(names => {
              for (let name of names) {
                console.debug('Deleting cache with name ', name);
                caches.delete(name);
              }
              console.debug('Phone-home - Service Worker has unregistered itself and destroyed old caches.');
            });
          }, 1000);
        }
        else {
          CACHE_ACTIVE = true;
          console.error('Phone-home - Service Worker could not unregister itself. It might be necessary to manually delete this Service Worker using browser devtools.');
        }
      })
      .catch(error => {
        CACHE_ACTIVE = true;
        console.error('Phone-home - ', error);
      });
  }

  // Set a very early fetch listener to be able to bypass all workbox logic while
  // the serviceworker is being uninstalled.
  self.addEventListener('fetch', event => {
    if (CACHE_ACTIVE === false) {
      console.debug('serviceworker is uninstalling, bypassing caching logic to prevent new cache entries from being saved.');
      event.respondWith(fetch(event.request));
    }
  })
  // Check whether the serviceworker should unregister itself.
  self.addEventListener('fetch', phoneHome);
  self.addEventListener('activate', phoneHome);

})(self, navigator, drupalPWASettings, pwaConsole);
