/**
 * @file
 *
 * To register, it is only necessary to call Notification.requestPermission(),
 * the rest is handled automatically on permission change.
 *
 * If permission is already granted, autoregistration behavior is handled
 * according to the configuration.
 */

(function (Drupal, $) {

  Drupal.pwa.webpush.ui = {
    'default': {text: Drupal.t('Enable notifications'), attr: {disabled: false, hidden: false}},
    'prompt': {text: Drupal.t('Enable notifications'), attr: {disabled: false, hidden: false}},
    'granted': {text: Drupal.t('Notifications enabled'), attr: {disabled: true, hidden: false}},
    'denied': {text: Drupal.t('Notifications blocked'), attr: {disabled: true, hidden: false}},
  };

  // Feature detection of push and notifications support
  if (!('PushManager' in window && 'Notification' in window)) {
    return;
  }

  var initialPermission = Notification.permission;

  // Update UI when the user changes permissions.
  navigator.permissions.query({name: 'notifications'}).then((result) => {
    result.addEventListener('change', updateUI);
  });

  /**
   * Show or hide the subscription button.
   *
   * The button is shown when:
   *  - Permission is "prompt".
   *
   * @param {Event} event
   */
  function updateUI(event) {
    var state = event.target.state;
    var $webpush = $('[data-drupal-pwa-webpush-register]');
    // Only show the element is the state is "prompt".
    $webpush.trigger('change:state', [state]);
  }

  /**
   * Apply UI changes to the register button.
   *
   * @param event
   * @param state
   */
  function changeState(event, state) {
    var ui = Drupal.pwa.webpush.ui[state];
    var element = event.target;
    Object.keys(ui.attr).forEach(function (attr) {
      element.toggleAttribute(attr, ui.attr[attr]);
    });
    element.innerText = ui.text;
  }

  /**
   * Triggers the authorisation popup.
   */
  function requestPermission() {
    Notification.requestPermission();
  }

  /**
   * When permission is granted but not autoregistred.
   *
   * @param event
   */
  function registerOnClick(event) {
    $(event.target).trigger('change:state', ['granted'])
    Drupal.pwa.webpush.subscribe()
      // There was an issue while subscribing, show the button.
      .catch(function () { $(event.target).trigger('change:state', ['prompt']) });
  }

  Drupal.behaviors.pwaWebPushRegister = {
    attach: function (context, settings) {
      var permission = Notification.permission;
      var $webpush = $(context).find('[data-drupal-pwa-webpush-register]')
        .once('webpush')
        // We only need to request the permission, the subscription is
        // automatically managed on permission change.
        .bind('click', requestPermission)
        // State is one of: default, prompt, denied, granted.
        .bind('change:state', changeState)
        // Initialize the button
        .trigger('change:state', [permission]);

      // Permission is granted but there are no subscriptions, show button.
      if ($webpush.length && initialPermission === 'granted') {
        Drupal.pwa.webpush.subscription.get().catch(function () {
          $webpush
            .trigger('change:state', ['prompt'])
            // If we click on the button, we trigger a subscription.
            .one('click', registerOnClick);
        });
      }

    }
  };


}(Drupal, jQuery));
