/**
 * @file
 *
 * Code to manage subscription. Reacts to permission changes only.
 * @todo use ES6 to make this readable.
 */

(function (Drupal, navigator, window, $) {
  'use strict';

  Drupal.pwa.webpush = {
    /**
     * Helper methods to check the browser status of the serviceworker.
     *
     * @namespace
     */
    subscription: {
      /**
       * Keep the current sha in memory to be able to remove it when permissions
       * are updated.
       *
       * @var {string}
       */
      sha256: null,
    },
    /**
     * Default notifications subscription options.
     */
    options: {
      userVisibleOnly: true,
    },
  };

  // Feature detection for push notifications.
  if (!('PushManager' in window && 'Notification' in window)) {
    return;
  }

  /**
   * Helper to build the URL to the Drupal backend to manage subscriptions.
   *
   * @param {string} sha
   */
  function registerUrl(sha) {
    return Drupal.settings.basePath + 'pwa/webpush/subscription/' + sha
  }

  /**
   * Helper to update Drupal.settings after a fetch.
   *
   * @param response
   * @returns {Promise<never>|Promise<*>}
   */
  function fetchCallback(response) {
    if (response.ok) {
      return response.json().then(Drupal.pwa.webpush.subscription.updateSettings);
    }
    return Promise.reject('Error: ' + response.status + ' ' + response.statusText);
  }

  /**
   * Takes care of subscribing the user to push notifications.
   */
  Drupal.pwa.webpush.subscribe = function () {
    var subscription = Drupal.pwa.webpush.subscription;

    if (Notification.permission !== 'granted') {
      return Promise.reject('Make sure permission for notification is granted before calling this function');
    }

    // Get any existing subscriptions.
    return subscription.get()
      // If there are no subscriptions, create one.
      // A subscription exists, check if Drupal knows about it.
      .then(subscription.check, subscription.create)
      // Drupal doesn't know about the subscription, save the subscription
      // data in Drupal.
      .catch(subscription.save);
  };

  /**
   * Return the list of subscriptions for the current user.
   */
  Drupal.pwa.webpush.subscription.list = function () {
    return Drupal.settings.pwa.webpush.userSubscriptions;
  };

  /**
   * Add a subscription to the list.
   */
  Drupal.pwa.webpush.subscription.list.add = function (json) {
    Drupal.settings.pwa.webpush.userSubscriptions[json.sid] = json.saved.endpoint_sha256;
  };

  /**
   * Remove a subscription from the list.
   */
  Drupal.pwa.webpush.subscription.list.remove = function (json) {
    delete Drupal.settings.pwa.webpush.userSubscriptions[json.sid];
  };

  /**
   *
   * @return {Promise<PushSubscription|boolean>}
   */
  Drupal.pwa.webpush.subscription.get = function () {
    return navigator.serviceWorker.ready
      .then(/** @return PushSubscription */function (registration) {
        return registration.pushManager.getSubscription();
      })
      .then(function (subscription) {
        // There is a subscription, check the validity.
        if (subscription) {
          return Promise.all([sha256(subscription.endpoint), subscription]);
        }
        return Promise.reject('No subscriptions');
      });
  };

  /**
   * Check that Drupal knows about this push registration.
   *
   * @param {Array<string|PushSubscription>} promises
   */
  Drupal.pwa.webpush.subscription.check = function (promises) {
    var sha = promises[0];
    var subscription = promises[1];
    var userSubs = Drupal.pwa.webpush.subscription.list();
    // Keep the sha in memory to be able to delete it when the user changes
    // permissions.
    Drupal.pwa.webpush.subscription.sha256 = sha;
    // The subscription is in the settings array, Drupal know about it.
    if (userSubs && Object.values(userSubs).includes(sha)) {
      console.debug('This serviceworker is already subscribed to push notifications.');
      return Promise.resolve([sha, subscription]);
    }
    return Promise.reject([sha, subscription]);
  };

  /**
   * @return {Promise<PushSubscription>}
   */
  Drupal.pwa.webpush.subscription.create = function () {
    var webpush = Drupal.pwa.webpush;
    var webpushSettings = Drupal.settings.pwa.webpush;
    var options = { applicationServerKey: webpushSettings.applicationServerKey };

    return navigator.serviceWorker.ready
      .then(function (registration) {
        return registration.pushManager.subscribe(Object.assign(options, webpush.options));
      })
      .then(webpush.subscription.get)
      // Save the subscription information to the Drupal backend.
      .then(webpush.subscription.save);
  };

  /**
   * Ajax call to store the value in the DB.
   */
  Drupal.pwa.webpush.subscription.save = function (promises) {
    var sha = promises[0];
    var subscription = promises[1];
    return fetch(registerUrl(sha), {
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body: JSON.stringify({ sub: subscription }),
      })
      .then(fetchCallback)
      .then(function () { return promises; });
  };

  /**
   * Delete a subscription from the database.
   *
   * This is not currently useful.
   */
  Drupal.pwa.webpush.subscription.delete = function (sha) {
    return fetch(registerUrl(sha), {
      method: 'DELETE'
    })
    .then(fetchCallback)
  };

  Drupal.pwa.webpush.subscription.updateSettings = function (json) {
    var sub = Drupal.pwa.webpush.subscription;
    if (json.status === 'ok') {
      if (json.action in sub.list) {
        sub.list[json.action](json);
      }
      if (json.action === 'add') {
        // Keep current sha in memory
        sub.sha256 = json.saved.endpoint_sha256;
      }
      else if (json.action === 'remove') {
        delete sub.sha256;
      }
    }
    return json;
  };

  // Check the subscription to fill in the sha256 variable on page load.
  Drupal.pwa.webpush.subscription.get().then(function (promises) {
    Drupal.pwa.webpush.subscription.sha256 = promises[0];
  }).catch(function () {
    // No subscriptions, do nothing.
  });

}(Drupal, navigator, window, jQuery));
